# Generated by Django 3.0.8 on 2020-09-08 10:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0020_auto_20200907_0753'),
    ]

    operations = [
        migrations.AddField(
            model_name='tag',
            name='description',
            field=models.TextField(blank=True, max_length=1024, null=True),
        ),
    ]
