{% extends "admin/base.html" %}

{% load i18n bootstrap4 %}

{% block head_title %}{% trans "Change Password" %}{% endblock %}

{% block breadcrumb %}
    <div class="col-sm-12 pl-0">
      <a href="{% url 'account_change_password' %}">{% trans "Change Password" %}</a>
    </div><!-- /.col -->
{% endblock %}

{% block content %}
    <div class="col-lg-12 col-md-12">
        <div class="p-1 p-md-3">
            <!-- form error messages -->
            <form id="changelist_form" class="change_password" method="POST" action="{% url 'account_change_password' %}">
              {% csrf_token %}
              {% bootstrap_form form %}
            </form>
        </div>
        <div class="p-1 p-md-3 bg-light text-dark">
          <ul class="list-unstyled">
            <li>
              <i class="fa fa-info-circle text-success m-2"></i>
              {% trans "Your password can’t be too similar to your other personal information." %}
            </li>
            <li>
              <i class="fa fa-info-circle text-success m-2"></i>
              {% trans "Your password must contain at least 8 characters." %}
            </li>
            <li>
              <i class="fa fa-info-circle text-success m-2"></i>
              {% trans "Your password can’t be a commonly used password." %}
            </li>
            <li>
              <i class="fa fa-info-circle text-success m-2"></i>
              {% trans "Your password can’t be entirely numeric." %}
            </li>
          </ul>
        </div>
    </div>
{% endblock %}


{% block widgetsbar %}
  <div class="card">
    <div class="card-body">
      <ul class="collection">
        <li class="collection-item d-flex flex-row-reverse mt-3">
            <button form="changelist_form" class="btn btn-primary btn-flat mx-1 btn-block" type="submit">
              <i class="fa fa-check"></i>
              {% trans "Submit" %}
            </button>
        </li>
        <li class="collection-item d-flex flex-row-reverse mt-3">
          <a class="btn btn-flat btn-block btn-light" href="{% url 'admin:index' %}">{% trans "Cancel" %}</a>
        </li>
      </ul>
    </div>
  </div>
{% endblock %}