# Generated by Django 3.1.3 on 2020-12-01 11:16

from django.db import migrations, models
import papermerge.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0033_auto_20201119_0032'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document',
            name='file_orig',
        ),
        migrations.AlterField(
            model_name='basetreenode',
            name='title',
            field=models.CharField(max_length=200, validators=[papermerge.core.validators.safe_character_validator], verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='document',
            name='file_name',
            field=models.CharField(default='', max_length=1024, validators=[papermerge.core.validators.safe_character_validator]),
        ),
        migrations.AlterField(
            model_name='document',
            name='notes',
            field=models.TextField(blank=True, null=True, validators=[papermerge.core.validators.safe_character_validator], verbose_name='Notes'),
        ),
        migrations.AlterField(
            model_name='document',
            name='text',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='tag',
            name='name',
            field=models.CharField(max_length=100, validators=[papermerge.core.validators.safe_character_validator], verbose_name='name'),
        ),
    ]
