# Generated by Django 3.0.2 on 2020-01-16 10:50
from django.db import migrations
#from papermerge.core.utils import get_sql_content


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]
    # DB dependent oprations moved out as plugin
    operations = [
#       migrations.RunSQL(
#           get_sql_content('01_triggers.sql')
#       ),
#       migrations.RunSQL(
#           get_sql_content('02_basetreenode.sql')
#       ),
#       migrations.RunSQL(
#           get_sql_content('03_update_lang_cols.sql')
#       ),
#       migrations.RunSQL(
#           get_sql_content('04_views.sql')
#       ),
    ]
