{% extends "account/base.html" %}

{% load i18n bootstrap4 %}
{% block head_title %}{% trans "Change Password" %}{% endblock %}

{% block content %}
<div class="login-box">
  <!-- /.login-logo -->
  <div class="card">
    <div class="card-body login-card-body">
        <p class="login-box-msg">{% if token_fail %}{% trans "Bad Token" %}{% else %}{% trans "Change Password" %}{% endif %}</p>
        {% if token_fail %}
            {% url 'account_reset_password' as passwd_reset_url %}
            <p>{% blocktrans %}The password reset link was invalid, possibly because it has already been used.  Please request a <a href="{{ passwd_reset_url }}">new password reset</a>.{% endblocktrans %}</p>
        {% else %}
            {% if form %}
                <form method="POST" action="{{ action_url }}">
                    {% csrf_token %}
                    {% bootstrap_form form %}
                    <input class="btn btn-block btn-success" type="submit" name="action" value="{% trans 'change password' %}"/>
                </form>
            {% else %}
                <p>{% trans 'Your password is now changed.' %}</p>
            {% endif %}
        {% endif %}
        </div> <!-- card body -->
    </div> <!-- card -->
</div> <!-- login box -->
{% endblock %}
