{% load static i18n admin_tags  %}

<!-- Brand Logo -->
<a href="{% url 'admin:index' %}" class="brand-link">
  <img src="{% static 'admin/img/logo_bbg.svg' %}" alt="" class="brand-image"
       style="opacity: .8">
  <span class="brand-text font-weight-light">Papermerge</span>
</a>

<!-- Sidebar -->
<div class="sidebar sidebar_collapse_remember">
  <!-- Sidebar Menu -->
  <nav class="mt-2">
    <ul class="nav nav-pills nav-sidebar flex-column" data-widget="treeview" role="menu" data-accordion="false">
      <li class="nav-item">
        <a href="{% url 'admin:browse' %}" class="nav-link {% activate_on 'index, browse' %}">
          <i class="nav-icon fa fa-folder text-warning"></i>
          <p>
            {% trans "Documents" %}
          </p>
        </a>
        <ul id="pinned_tags_view" class="pinned_tags">
          {% for tag in pinned_tags %}
            <li class="my-1">
              <a class="tag" href="{% url 'admin:browse' %}#tagged/{{tag.name}}" style="background-color:{{tag.bg_color}}; color: {{tag.fg_color}};"
              >{{tag.name}}</a>
            </li>
          {% endfor %}
        </ul>
      </li>
  
      <li class="nav-item">
        <a href="{% url 'admin:inbox' %}" class="nav-link  {% activate_on 'inbox' %}">
          <i class="nav-icon fa fa-inbox"></i>
          <p>
            {% trans "Inbox" %} ({{ inbox_count }})
          </p>
        </a>
      </li>
      {% for item in sidebar_menu %}
          <li class="nav-item">
              <a href="{{item.url}}" class="nav-link {% activate_on item.activate_on %}">
                  <i class="nav-icon fas {{item.css_icon_class}}"></i> 
                  <p>{{item.title}}</p>
              </a>
          </li>
      {% endfor %}
        {% for nav_item in leftside_navigation %}
          <li class="nav-item">
            <a href="{{ nav_item.url }}" class="nav-link {% activate_on nav_item.activate_on %}">
              <i class="nav-icon {{nav_item.icon_css_class}}"></i>
              <p>
                {{nav_item.title}}
              </p>
            </a>
          </li>
        {% endfor %}
    </ul>
  </nav>
  <!-- /.sidebar-menu -->
</div>
<!-- /.sidebar -->