# Generated by Django 3.0.6 on 2020-06-02 11:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0008_auto_20200601_1812'),
    ]

    operations = [
        migrations.CreateModel(
            name='KVStore',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(blank=True, max_length=200, null=True)),
                ('namespace', models.CharField(blank=True, max_length=1024, null=True)),
                ('namespaced_key', models.CharField(blank=True, max_length=200, null=True)),
                ('value', models.TextField(blank=True, null=True)),
                ('human_key', models.CharField(blank=True, max_length=200, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='KVStorePage',
            fields=[
                ('kvstore_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.KVStore')),
                ('page_id', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Page')),
            ],
            bases=('core.kvstore',),
        ),
        migrations.CreateModel(
            name='KVStoreNode',
            fields=[
                ('kvstore_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.KVStore')),
                ('node', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='kvstore', to='core.BaseTreeNode')),
            ],
            bases=('core.kvstore',),
        ),
    ]
