# Generated by Django 3.0.7 on 2020-07-21 13:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0017_automate_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='automate',
            name='plugin_name',
            field=models.CharField(blank=True, choices=[], default=None, max_length=256, null=True),
        ),
    ]
